@@ -31,6 +31,7 @@
 #include "seahorse-util.h"
 #include "seahorse-pgp-key.h"
 #include "seahorse-gconf.h"
+#include "seahorse-vfs-data.h"
 
 #ifdef WITH_HKP
 
@@ -127,7 +128,7 @@ dump_soup_request (SoupMessage *msg)
     
     if (msg->request.body) {
         t = g_strndup (msg->request.body, msg->request.length);
-        g_printerr ("request: %s\n", t);
+        g_printerr ("request: %s\n", t ? t : "");
         g_free (t);
     }    
 }
@@ -143,7 +144,7 @@ dump_soup_response (SoupMessage *msg)
     
     if (msg->response.body) {
         t = g_strndup (msg->response.body, msg->response.length);
-        g_printerr ("response: %s\n", t);
+        g_printerr ("response: %s\n", t ? t : "");
         g_free (t);
     }    
 }
@@ -168,6 +169,7 @@ DECLARE_OPERATION (HKP, hkp)
     SoupSession *session;           /* The HTTP session */
     guint total;                    /* Number of requests queued */
     guint requests;                 /* Number of requests remaining */
+    gboolean cancelling;            /* Cancelling the request, don't process */
 END_DECLARE_OPERATION
 
 IMPLEMENT_OPERATION (HKP, hkp)
@@ -250,9 +252,12 @@ seahorse_hkp_operation_cancel (SeahorseOperation *operation)
     g_assert (SEAHORSE_IS_HKP_OPERATION (operation));
     hop = SEAHORSE_HKP_OPERATION (operation);
     
+    g_return_if_fail (seahorse_operation_is_running (operation));
+    hop->cancelling = TRUE;
+    
     if (hop->session != NULL) 
         soup_session_abort (hop->session);
-
+    
     seahorse_operation_mark_done (operation, TRUE, NULL);
 }
 
@@ -331,6 +336,9 @@ fail_hkp_operation (SeahorseHKPOperation *hop, SoupMessage *msg, const gchar *te
 {
     gchar *t, *server;
     GError *error = NULL;
+    
+    if (!seahorse_operation_is_running (SEAHORSE_OPERATION (hop)))
+        return;
 
     g_object_get (hop->hsrc, "key-server", &server, NULL);
 
@@ -527,6 +535,9 @@ refresh_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     GList *keys, *k;
     gchar *t;
     
+    if (hop->cancelling)
+        return;
+    
     DEBUG_HKP (("[hkp] Search Result:\n"));
     DEBUG_RESPONSE (msg);
     
@@ -550,8 +561,8 @@ refresh_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
     else
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Searching for keys..."), 
-                                          hop->requests, hop->total);
+        seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (hop), _("Searching for keys..."), 
+                                               hop->requests, hop->total);
 }
 
 static gchar*
@@ -563,7 +574,10 @@ get_send_result (const gchar *response, unsigned int length)
     gboolean is_error = FALSE;
     
     text = g_strndup (response, length);
-    lines = g_strsplit (text, "\n", 0);    
+    if (!text)
+        return g_strdup ("");
+    
+    lines = g_strsplit (text, "\n", 0); 
 
     for (l = lines; *l; l++) {
 
@@ -599,6 +613,9 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
 {
     gchar *errmsg;
 
+    if (hop->cancelling)
+        return;
+
     DEBUG_HKP (("[hkp] Send Result:\n"));
     DEBUG_RESPONSE (msg);
     
@@ -620,8 +637,8 @@ send_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
     else
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Uploading keys..."), 
-                                          hop->requests, hop->total);
+        seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (hop), _("Uploading keys..."), 
+                                               hop->requests, hop->total);
 }
 
 gboolean
@@ -650,12 +667,16 @@ detect_key (const gchar *text, gint len, const gchar **start, const gchar **end)
 static void 
 get_callback (SoupMessage *msg, SeahorseHKPOperation *hop) 
 {
+    GError *err = NULL;
     const gchar *start;
     const gchar *end;
     gpgme_data_t data;
     const gchar *text;
+    gboolean ret;
     guint len;
-    int r;
+    
+    if (hop->cancelling)
+        return;
     
     DEBUG_HKP (("[hkp] Get Result:\n"));
     DEBUG_RESPONSE (msg);
@@ -680,18 +701,20 @@ get_callback (SoupMessage *msg, SeahorseHKPOperation *hop)
         data = (gpgme_data_t)seahorse_operation_get_result (SEAHORSE_OPERATION (hop));
         g_return_if_fail (data != NULL);
             
-        r = gpgme_data_write (data, start, end - start);
-        g_return_if_fail (r != -1);
-
-        r = gpgme_data_write (data, "\n", 1);
-        g_return_if_fail (r != -1);
+        ret = seahorse_vfs_data_write_all (data, start, end - start, &err) &&
+              seahorse_vfs_data_write_all (data, "\n", -1, &err);
+        
+        if (!ret) {
+            seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, err);
+            return;
+        }
     }
         
     if (--hop->requests <= 0)
         seahorse_operation_mark_done (SEAHORSE_OPERATION (hop), FALSE, NULL);
     else
-        seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), _("Retrieving keys..."), 
-                                          hop->requests, hop->total);
+        seahorse_operation_mark_progress_full (SEAHORSE_OPERATION (hop), _("Retrieving keys..."), 
+                                               hop->requests, hop->total);
 }
 
 /* -----------------------------------------------------------------------------
@@ -704,12 +727,12 @@ static void seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass);
 /* SeahorseKeySource methods */
 static SeahorseOperation*  seahorse_hkp_source_load       (SeahorseKeySource *src,
                                                            SeahorseKeySourceLoad load,
+                                                           GQuark keyid,
                                                            const gchar *key);
 static SeahorseOperation*  seahorse_hkp_source_import     (SeahorseKeySource *sksrc, 
                                                            gpgme_data_t data);
-static SeahorseOperation*  seahorse_hkp_source_export     (SeahorseKeySource *sksrc, 
-                                                           GList *keys,     
-                                                           gboolean complete,
+static SeahorseOperation*  seahorse_hkp_source_export_raw (SeahorseKeySource *sksrc, 
+                                                           GSList *keys,     
                                                            gpgme_data_t data);
 
 static SeahorseKeySourceClass *parent_class = NULL;
@@ -743,14 +766,14 @@ seahorse_hkp_source_class_init (SeahorseHKPSourceClass *klass)
     key_class = SEAHORSE_KEY_SOURCE_CLASS (klass);
     key_class->load = seahorse_hkp_source_load;
     key_class->import = seahorse_hkp_source_import;
-    key_class->export = seahorse_hkp_source_export;
+    key_class->export_raw = seahorse_hkp_source_export_raw;
 
     parent_class = g_type_class_peek_parent (klass);
 }
 
 static SeahorseOperation*
 seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
-                          const gchar *match)
+                          GQuark keyid, const gchar *match)
 {
     SeahorseOperation *op;
     SeahorseHKPOperation *hop;
@@ -761,7 +784,7 @@ seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     g_assert (SEAHORSE_IS_KEY_SOURCE (src));
     g_assert (SEAHORSE_IS_HKP_SOURCE (src));
     
-    op = parent_class->load (src, load, match);
+    op = parent_class->load (src, load, keyid, match);
     if (op != NULL)
         return op;
 
@@ -775,7 +798,7 @@ seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     /* Load a specific key */
     else if(load == SKSRC_LOAD_KEY) 
         /* TODO: Does this actually work? */
-        pattern = soup_uri_encode (match, NULL);    
+        pattern = soup_uri_encode (seahorse_key_get_rawid (keyid), NULL);    
 
     g_return_val_if_fail (pattern != NULL, NULL);
     
@@ -799,7 +822,7 @@ seahorse_hkp_source_load (SeahorseKeySource *src, SeahorseKeySourceLoad load,
     
     hop->total = hop->requests = 1;
     t = g_strdup_printf (_("Searching for keys on: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0, hop->total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0.0);
     g_free (t);
 
     g_free (server);    
@@ -872,32 +895,34 @@ seahorse_hkp_source_import (SeahorseKeySource *sksrc, gpgme_data_t data)
 
     hop->total = hop->requests;
     t = g_strdup_printf (_("Connecting to: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0, hop->total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0.0);
     g_free (t);
 
     g_free (server);
     g_free (uri);
     
     seahorse_util_string_slist_free (keydata);
+    gpgmex_data_release (data);
     
     return SEAHORSE_OPERATION (hop);
 }
 
-static SeahorseOperation* 
-seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys, 
-                            gboolean complete, gpgme_data_t data)
+static SeahorseOperation*  
+seahorse_hkp_source_export_raw (SeahorseKeySource *sksrc, GSList *keyids,
+                                gpgme_data_t data)
 {
     SeahorseHKPOperation *hop;
     SeahorseHKPSource *hsrc;
     SoupMessage *message;
     gchar *t, *server, *uri;
     const gchar *fpr;
-    guint l;
+    guint len;
+    GSList *l;
     
     g_return_val_if_fail (SEAHORSE_IS_HKP_SOURCE (sksrc), NULL);
     hsrc = SEAHORSE_HKP_SOURCE (sksrc);
     
-    if (g_list_length (keys) == 0)
+    if (g_slist_length (keyids) == 0)
         return seahorse_operation_new_complete (NULL);
 
     server = get_http_server_address (sksrc);
@@ -911,20 +936,18 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
         seahorse_operation_mark_result (SEAHORSE_OPERATION (hop), data, NULL);
     } else {
         /* But when we auto create a data object then we free it */
-        gpgme_data_new (&data);
-        g_return_val_if_fail (data != NULL, NULL);
+        data = gpgmex_data_new ();
         seahorse_operation_mark_result (SEAHORSE_OPERATION (hop), data, 
-                                        (GDestroyNotify)gpgme_data_release);
+                                        (GDestroyNotify)gpgmex_data_release);
     }
     
-    for ( ; keys; keys = g_list_next (keys)) {
-        g_assert (SEAHORSE_IS_KEY (keys->data));
+    for (l = keyids; l; l = g_slist_next (l)) {
 
         /* Get the key id and limit it to 8 characters */
-        fpr = seahorse_key_get_keyid (SEAHORSE_KEY (keys->data));
-        l = strlen (fpr);
-        if (l > 8)
-            fpr += (l - 8);
+        fpr = (const char*)seahorse_key_get_rawid (GPOINTER_TO_UINT (l->data));
+        len = strlen (fpr);
+        if (len > 8)
+            fpr += (len - 8);
 
         /* The get key URI */
         uri = g_strdup_printf ("http://%s/pks/lookup?op=get&search=0x%s", 
@@ -944,7 +967,7 @@ seahorse_hkp_source_export (SeahorseKeySource *sksrc, GList *keys,
     
     hop->total = hop->requests;
     t = g_strdup_printf (_("Connecting to: %s"), server);
-    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0, hop->total);
+    seahorse_operation_mark_progress (SEAHORSE_OPERATION (hop), t, 0.0);
     g_free (t);
 
     g_free (server);
