@@ -132,7 +132,7 @@ watch_ssh_process (GPid pid, gint status, SeahorseSSHOperation *sop)
         /* Was killed */
         if (!WIFEXITED (status)) {
             seahorse_operation_mark_done (SEAHORSE_OPERATION (sop), FALSE, 
-                g_error_new (SEAHORSE_ERROR, 0, _("The SSH command was terminated unexpectedly.")));
+                g_error_new (SEAHORSE_ERROR, 0, _("The ssh command was terminated unexpectedly.")));
             
         /* Command failed */
         } else if (WEXITSTATUS (status) != 0) {
@@ -295,23 +295,37 @@ askpass_handler (GIOChannel *source, GIOCondition condition, SeahorseSSHOperatio
             
             if (g_ascii_strncasecmp (COMMAND_PASSWORD, string, COMMAND_PASSWORD_LEN) == 0) {
                 line = g_strstrip (string + COMMAND_PASSWORD_LEN);
-                if (pv->password_cb)
+                
+                /* Prompt for a password */
+                if (pv->password_cb) {
                     result = (pv->password_cb) (sop, line);
+                    
+                    /* Cancelled prompt, cancel operation */
+                    if (!result) {
+                        if (seahorse_operation_is_running (SEAHORSE_OPERATION (sop)))
+                            seahorse_operation_cancel (SEAHORSE_OPERATION (sop));
+                        DEBUG_OPERATION (("SSHOP: password prompt cancelled\n"));
+                        ret = FALSE;
+                    }
+                }
+                
                 pv->prompt_requests++;
             }
             
-            /* And write the result back out to seahorse-ssh-askpass */
-            DEBUG_OPERATION (("SSHOP: seahorse-ssh-askpass response: %s\n", result ? result : ""));
-            if (result)
-                g_io_channel_write_chars (pv->io_askpass, result, strlen (result), &length, &err);
-            if (err == NULL)
-                g_io_channel_write_chars (pv->io_askpass, "\n", 1, &length, &err);
-            if (err == NULL)
-                g_io_channel_flush (pv->io_askpass, &err);
-            if (err != NULL) {
-                g_critical ("couldn't read from seahorse-ssh-askpass: %s", err->message);
-                g_clear_error (&err);
-                ret = FALSE;
+            if (ret) {
+                /* And write the result back out to seahorse-ssh-askpass */
+                DEBUG_OPERATION (("SSHOP: seahorse-ssh-askpass response: %s\n", result ? result : ""));
+                if (result)
+                    g_io_channel_write_chars (pv->io_askpass, result, strlen (result), &length, &err);
+                if (err == NULL)
+                    g_io_channel_write_chars (pv->io_askpass, "\n", 1, &length, &err);
+                if (err == NULL)
+                    g_io_channel_flush (pv->io_askpass, &err);
+                if (err != NULL) {
+                    g_critical ("couldn't read from seahorse-ssh-askpass: %s", err->message);
+                    g_clear_error (&err);
+                    ret = FALSE;
+                }
             }
         }
     }
@@ -424,7 +438,7 @@ prompt_passphrase (SeahorseSSHOperation *sop, const gchar* title, const gchar* m
     if (pv->prompt_skey)
         display = seahorse_key_get_display_name (pv->prompt_skey);
     else 
-        display = g_strdup (_("SSH key"));
+        display = g_strdup (_("Secure Shell key"));
     msg = g_strdup_printf (message, display);
     g_free (display);
 
@@ -680,14 +694,14 @@ seahorse_ssh_operation_sync (SeahorseSSHSource *ssrc, const gchar *command,
     
     if (!WIFEXITED (status)) {
         g_critical ("SSH command didn't exit properly: %s", command);
-        g_set_error (error, SEAHORSE_ERROR, 0, "%s", _("The SSH command was terminated unexpectedly."));
+        g_set_error (error, SEAHORSE_ERROR, 0, "%s", _("The ssh command was terminated unexpectedly."));
         g_free (sout);
         g_free (serr);
         return NULL;
     }
     
     if (WEXITSTATUS (status) != 0) {
-        g_set_error (error, SEAHORSE_ERROR, 0, "%s", _("The SSH command failed."));
+        g_set_error (error, SEAHORSE_ERROR, 0, "%s", _("The ssh command failed."));
         g_warning ("SSH command failed: %s (%d)", command, WEXITSTATUS (status));
         if (serr && serr[0])
             g_warning ("SSH error output: %s", serr);
@@ -710,7 +724,7 @@ upload_password_cb (SeahorseSSHOperation *sop, const gchar* msg)
     DEBUG_OPERATION (("in upload_password_cb\n"));
 
     /* Just prompt over and over again */
-    return prompt_passphrase (sop, _("SSH Passphrase"), msg, NULL, FALSE);
+    return prompt_passphrase (sop, _("Secure Shell Passphrase"), msg, NULL, FALSE);
 }
 
 SeahorseOperation*  
@@ -873,7 +887,7 @@ generate_password_cb (SeahorseSSHOperation *sop, const gchar* msg)
 
     /* If the first time then prompt */
     if (!pv->prompt_dialog) {
-        return prompt_passphrase (sop, _("New SSH Passphrase"), 
+        return prompt_passphrase (sop, _("New Secure Shell Key Passphrase"), 
                 _("Enter a passphrase for your new SSH key."), NULL, TRUE);
     }
     
@@ -1012,7 +1026,7 @@ load_password_cb (SeahorseSSHOperation *sop, const gchar* msg)
     }
 #endif /* WITH_GNOME_KEYRING */
     
-    return prompt_passphrase (sop, _("SSH Key Passphrase"), _("Enter the SSH passphrase for: %s"), 
+    return prompt_passphrase (sop, _("Secure Shell Key Passphrase"), _("Enter the passphrase for: %s"), 
                               _("Save this passphrase in my keyring"), FALSE);
 }
 
@@ -1221,3 +1235,67 @@ seahorse_ssh_operation_authorize (SeahorseSSHSource *ssrc, SeahorseSSHKey *skey,
     
     return seahorse_operation_new_complete (err);
 }
+
+/* -----------------------------------------------------------------------------
+ * RENAME COMMENT ON A KEY 
+ */
+
+static gboolean
+change_raw_comment (SeahorseSSHKeyData *keydata, const gchar *newcomment)
+{
+    const gchar *x = keydata->rawdata;
+    gchar *result;
+    gchar **parts;
+    
+    g_assert (x);
+    while (*x && g_ascii_isspace (*x))
+        ++x;
+    
+    parts = g_strsplit_set (x, " ", 3);
+    if (!parts[0] || !parts[1])
+        return FALSE;
+    
+    result = g_strconcat (parts[0], " ", parts[1], " ", newcomment, NULL);
+    g_strfreev(parts);
+    
+    g_free (keydata->rawdata);
+    keydata->rawdata = result;
+    return TRUE;
+}
+
+SeahorseOperation*
+seahorse_ssh_operation_rename (SeahorseSSHSource *ssrc, SeahorseSSHKey *skey,
+                               const gchar *newcomment)
+{
+    SeahorseSSHKeyData *keydata;
+    GError *err = NULL;
+    
+    g_return_val_if_fail (SEAHORSE_IS_SSH_SOURCE (ssrc), NULL);
+    g_return_val_if_fail (SEAHORSE_IS_SSH_KEY (skey), NULL);
+    g_assert (seahorse_ssh_key_data_is_valid (skey->keydata));
+    g_assert (skey->keydata->rawdata);
+    
+    keydata = seahorse_ssh_key_data_dup (skey->keydata);
+    
+    if (!newcomment)
+        newcomment = "";
+    
+    if (!change_raw_comment (keydata, newcomment ? newcomment : ""))
+        g_return_val_if_reached (NULL);
+    
+    DEBUG_OPERATION (("renaming key to: %s", newcomment));
+    
+    /* Just part of a file for this key */
+    if (keydata->partial) {
+        g_assert (keydata->pubfile);
+        seahorse_ssh_key_data_filter_file (keydata->pubfile, keydata, keydata, &err);
+        
+    /* A full file for this key */
+    } else {
+        g_assert (keydata->pubfile);
+        seahorse_util_write_file_private (keydata->pubfile, keydata->rawdata, &err);
+    }
+    
+    seahorse_ssh_key_data_free (keydata);
+    return seahorse_operation_new_complete (err);
+}
